@@ -28,6 +28,7 @@
 
 #include "ApvlvUtil.hpp"
 #include "ApvlvParams.hpp"
+#include "ApvlvInfo.hpp"
 #include "ApvlvView.hpp"
 #include "ApvlvDoc.hpp"
 
@@ -48,7 +49,7 @@ namespace apvlv
 {
   static GtkPrintSettings *settings = NULL;
 
-  const int APVLV_DOC_CURSOR_WIDTH = 10;
+  const int APVLV_DOC_CURSOR_WIDTH = 2;
 
     ApvlvDoc::ApvlvDoc (int w, int h, const char *zm, bool cache)
   {
@@ -124,10 +125,7 @@ namespace apvlv
     if (mCurrentCache2)
       delete mCurrentCache2;
 
-    if (mFilestr != helppdf)
-      {
-	savelastposition ();
-      }
+    savelastposition (filename ());
     mPositions.clear ();
 
     delete mStatus;
@@ -136,7 +134,7 @@ namespace apvlv
   void ApvlvDoc::blankarea (int x1, int y1, int x2, int y2, guchar * buffer,
 			    int width, int height)
   {
-    debug ("x1: %d, y1: %d, x2: %d, y2:%d", x1, y1, x2, y2);
+//    debug ("x1: %d, y1: %d, x2: %d, y2:%d", x1, y1, x2, y2);
     if (x2 > width)
       {
 	x2 = width;
@@ -158,9 +156,94 @@ namespace apvlv
       }
   }
 
+  void ApvlvDoc::blankaction (double x, double y)
+  {
+    ApvlvDocCache *cache;
+    ApvlvPos pos = { x, x, y, y };
+
+    if (x < 0)
+      {
+	x = 0;
+      }
+    if (y < 0)
+      {
+	y = 0;
+      }
+
+    cache = mCurrentCache1;
+    if (x > cache->getwidth ())
+      {
+	x = cache->getwidth ();
+      }
+
+    if (y > cache->getheight ())
+      {
+	if (mCurrentCache2 != NULL)
+	  {
+	    cache = mCurrentCache2;
+	  }
+	else
+	  {
+	    cache = NULL;
+	  }
+      }
+
+    if (cache == NULL)
+      {
+	return;
+      }
+
+    if (strcasecmp (gParams->values ("doubleclick"), "word") == 0)
+      {
+	ApvlvWord *word;
+	word =
+	  cache->getword (x,
+			  cache ==
+			  mCurrentCache2 ? y - mCurrentCache1->getheight () -
+			  gParams->valuei ("continuouspad") : y);
+	if (word != NULL)
+	  {
+	    pos = word->pos;
+	    debug ("get word: [%s] x1:%f, x2:%f, y1:%f, y2:%f",
+		   word->word.c_str (), pos.x1, pos.x2, pos.y1, pos.y2);
+	  }
+      }
+    else if (strcasecmp (gParams->values ("doubleclick"), "line") == 0)
+      {
+	ApvlvLine *line;
+	line =
+	  cache->getline (x,
+			  cache ==
+			  mCurrentCache2 ? y - mCurrentCache1->getheight () -
+			  gParams->valuei ("continuouspad") : y);
+	if (line != NULL)
+	  {
+	    pos = line->pos;
+	  }
+      }
+    else if (strcasecmp (gParams->values ("doubleclick"), "page") == 0)
+      {
+	pos.x1 = 0;
+	pos.x2 = cache->getwidth ();
+	pos.y1 = 0;
+	pos.y2 = cache->getheight ();
+      }
+    else
+      {
+	return;
+      }
+
+    mInVisual = VISUAL_CTRL_V;
+    mBlankx1 = pos.x1;
+    mBlanky1 = pos.y1;
+    blank (pos.x2, pos.y2);
+
+//    yank (1);
+  }
+
   void ApvlvDoc::blank (int bx, int by)
   {
-    debug ("bx: %d, by: %d", bx, by);
+//    debug ("bx: %d, by: %d", bx, by);
 
     ApvlvDocCache *cache = mCurrentCache1;
     gint rate = cache->getheight () / mLines;
@@ -250,31 +333,42 @@ namespace apvlv
 
     if (mInVisual == VISUAL_V)
       {
-	debug ("");
-	if (y1 == y2)
+//      debug ("");
+	if (y1 + rate >= y2)
 	  {
-	    blankarea (x1, y1, x2 + APVLV_DOC_CURSOR_WIDTH, y2 + rate,
+	    blankarea (x1, y1, x2 + APVLV_DOC_CURSOR_WIDTH,
+		       y1 == y2 ? y1 + rate : y2, buffer, cache->getwidth (),
+		       cache->getheight ());
+	  }
+	else if (y1 + rate * 2 >= y2)
+	  {
+	    blankarea (x1, y1, cache->getwidth (), y1 + rate, buffer,
+		       cache->getwidth (), cache->getheight ());
+	    blankarea (0, y1 + rate, x2 + APVLV_DOC_CURSOR_WIDTH, y2,
 		       buffer, cache->getwidth (), cache->getheight ());
 	  }
 	else
 	  {
 	    blankarea (x1, y1, cache->getwidth (), y1 + rate, buffer,
 		       cache->getwidth (), cache->getheight ());
-	    blankarea (0, y1 + rate, cache->getwidth (), y2, buffer,
+	    blankarea (0, y1 + rate, cache->getwidth (), y2 - rate, buffer,
 		       cache->getwidth (), cache->getheight ());
-	    blankarea (0, y2, x2 + APVLV_DOC_CURSOR_WIDTH, y2 + rate,
+	    blankarea (0, y2 - rate, x2 + APVLV_DOC_CURSOR_WIDTH, y2,
 		       buffer, cache->getwidth (), cache->getheight ());
 	  }
       }
     else if (mInVisual == VISUAL_CTRL_V)
       {
-	debug ("");
-	blankarea (x1, y1, x2 + APVLV_DOC_CURSOR_WIDTH, y2 + rate, buffer,
+//      debug ("");
+	blankarea (x1, y1,
+		   x1 + APVLV_DOC_CURSOR_WIDTH >
+		   x2 ? x1 + APVLV_DOC_CURSOR_WIDTH : x2,
+		   y1 + rate > y2 ? y1 + rate : y2, buffer,
 		   cache->getwidth (), cache->getheight ());
       }
     else
       {
-	debug ("");
+//      debug ("");
 	blankarea (bx, by, bx + APVLV_DOC_CURSOR_WIDTH, by + rate, buffer,
 		   cache->getwidth (), cache->getheight ());
       }
@@ -601,35 +695,25 @@ namespace apvlv
       }
   }
 
-  bool ApvlvDoc::savelastposition ()
+  bool ApvlvDoc::savelastposition (const char *filename)
   {
-    if (filename () == NULL || gParams->valueb ("noinfo"))
+    if (filename == NULL || helppdf == filename || gParams->valueb ("noinfo"))
       {
 	return false;
       }
 
-    gchar *path = absolutepath (sessionfile.c_str ());
-    ofstream os (path, ios::app);
-    g_free (path);
+    bool ret = gInfo->file (mPagenum, scrollrate (), filename);
 
-    if (os.is_open ())
-      {
-	os << ">";
-	os << filename () << "\t";
-	os << mPagenum << "\t";
-	os << scrollrate ();
-	os << "\n";
-	os.close ();
-	return true;
-      }
-    return false;
+    return ret;
   }
 
-  bool ApvlvDoc::loadlastposition ()
+  bool ApvlvDoc::loadlastposition (const char *filename)
   {
-    bool ret = false;
-    int pagen = 0;
-    double scrollr = 0.00;
+    if (filename == NULL || helppdf == filename || gParams->valueb ("noinfo"))
+      {
+	showpage (0);
+	return false;
+      }
 
     if (gParams->valueb ("noinfo"))
       {
@@ -637,38 +721,14 @@ namespace apvlv
 	return false;
       }
 
-    char *path = absolutepath (sessionfile.c_str ());
-    ifstream os (path, ios::in);
-    g_free (path);
+    bool ret = false;
 
-    if (os.is_open ())
-      {
-	string line;
+    infofile *fp = gInfo->file (filename);
 
-	while ((getline (os, line)) != NULL)
-	  {
-	    const char *p = line.c_str ();
-
-	    if (*p == '>')
-	      {
-		stringstream ss (++p);
-
-		string files;
-		ss >> files;
-
-		if (files == mFilestr)
-		  {
-		    ss >> pagen >> scrollr;
-		    ret = true;
-		  }
-	      }
-	  }
-	os.close ();
-      }
 
     // correctly check
-    mScrollvalue = scrollr >= 0 ? scrollr : 0;
-    showpage (pagen >= 0 ? pagen : 0);
+    mScrollvalue = fp->rate;
+    showpage (fp->page);
 
     // Warning
     // I can't think a better way to scroll correctly when
@@ -680,7 +740,7 @@ namespace apvlv
 
   bool ApvlvDoc::reload ()
   {
-    savelastposition ();
+    savelastposition (filename ());
     return loadfile (mFilestr, false);
   }
 
@@ -689,6 +749,25 @@ namespace apvlv
     return loadfile (filename.c_str (), check);
   }
 
+  gint ApvlvDoc::pagenumber ()
+  {
+    if (mContinuous && mCurrentCache2 != NULL)
+      {
+	if (scrollrate () > 0.5)
+	  {
+	    return mCurrentCache2->getpagenum () + 1;
+	  }
+	else
+	  {
+	    return mCurrentCache1->getpagenum () + 1;
+	  }
+      }
+    else
+      {
+	return mPagenum + 1;
+      }
+  }
+
   bool ApvlvDoc::usecache ()
   {
     return false;
@@ -698,8 +777,8 @@ namespace apvlv
   {
     if (use)
       {
-	warnp ("No pthread, can't support cache!!!");
-	warnp
+	gView->errormessage ("No pthread, can't support cache!!!");
+	gView->infomessage
 	  ("If you really have pthread, please recomplie the apvlv and try again.");
       }
   }
@@ -751,13 +830,21 @@ namespace apvlv
 	    mCurrentCache2 = new ApvlvDocCache (mFile);
 	  }
 
-	loadlastposition ();
+	loadlastposition (filename);
 
 	mStatus->show ();
 
 	setactive (true);
 
 	mReady = true;
+
+	mSearchStr = "";
+	if (mSearchResults != NULL)
+	  {
+	    delete mSearchResults;
+	    mSearchResults = NULL;
+	  }
+
 	mInVisual = VISUAL_NONE;
       }
 
@@ -1041,7 +1128,7 @@ namespace apvlv
     gint opage = mPagenum, npage = mPagenum;
 
     gint ny1 = mCury + mVrate * times;
-    if (ny1 - mVaj->value > mVaj->page_size)
+    if (ny1 - mVaj->value >= mVaj->page_size)
       {
 	ApvlvCore::scrolldown (times);
 	npage = mPagenum;
@@ -1162,11 +1249,16 @@ namespace apvlv
       }
   }
 
-  void ApvlvDoc::search (const char *str, bool reverse)
+  bool ApvlvDoc::search (const char *str, bool reverse)
   {
+    if (*str == '\0' && mSearchStr == "")
+      {
+	return false;
+      }
+
     if (!needsearch (str, reverse))
       {
-	return;
+	return true;
       }
 
     if (mSearchResults != NULL)
@@ -1192,7 +1284,7 @@ namespace apvlv
 	      {
 		showpage ((i + sum) % sum, 0.5);
 		markselection ();
-		break;
+		return true;
 	      }
 	  }
 
@@ -1200,7 +1292,7 @@ namespace apvlv
 
 	if (!reverse && i < (wrap ? (from + sum) : (sum - 1)))
 	  {
-	    debug ("wrap: %d, i++:", wrap, i, i + 1);
+//          debug ("wrap: %d, i++:", wrap, i, i + 1);
 	    i++;
 	  }
 	else if (reverse && i > (wrap ? (from - sum) : 0))
@@ -1210,7 +1302,9 @@ namespace apvlv
 	  }
 	else
 	  {
-	    break;
+	    gView->errormessage ("can't find word: '%s'",
+				 mSearchStr.c_str ());
+	    return false;
 	  }
       }
   }
@@ -1245,7 +1339,7 @@ namespace apvlv
 
     if (ct % 90 != 0)
       {
-	warnp ("Not a 90 times value, ignore.");
+	gView->errormessage ("Not a 90 times value: %d", ct);
 	return false;
       }
 
@@ -1431,10 +1525,26 @@ namespace apvlv
       {
 	if (button->type == GDK_BUTTON_PRESS)
 	  {
-	    doc->mBlankx1 = button->x;
-	    doc->mBlanky1 = button->y;
-	    doc->mInVisual = VISUAL_NONE;
-	    doc->blank (button->x, button->y);
+	    // this is a manual method to test double click
+	    // I think, a normal user double click will in 500 millseconds
+	    if (button->time - doc->mLastpress < 500)
+	      {
+		doc->mInVisual = VISUAL_NONE;
+		double rx, ry;
+		doc->eventpos (button->x, button->y, &rx, &ry);
+		doc->blankaction (rx, ry);
+	      }
+	    else
+	      {
+		doc->mBlankx1 = button->x;
+		doc->mBlanky1 = button->y;
+		doc->mInVisual = VISUAL_NONE;
+		double rx, ry;
+		doc->eventpos (button->x, button->y, &rx, &ry);
+		doc->blank (rx, ry);
+	      }
+
+	    doc->mLastpress = button->time;
 	  }
       }
     else if (button->button == 3)
@@ -1467,6 +1577,8 @@ namespace apvlv
 					    ApvlvDoc * doc)
   {
     doc->yank (1);
+    doc->mInVisual = VISUAL_NONE;
+    doc->blank (doc->mCurx, doc->mCury);
   }
 
   void
@@ -1474,13 +1586,42 @@ namespace apvlv
 				      GdkEventMotion * motion, ApvlvDoc * doc)
   {
     doc->mInVisual = VISUAL_V;
-    doc->blank (motion->x, motion->y);
+    double rx, ry;
+    doc->eventpos (motion->x, motion->y, &rx, &ry);
+    doc->blank (rx, ry);
+  }
+
+  void ApvlvDoc::eventpos (double x, double y, double *rx, double *ry)
+  {
+    int dw, dh;
+    if (rx != NULL)
+      {
+	dw = mPagex * mZoomrate - mScrollwin->allocation.width;
+	dw = dw >> 1;
+	if (dw >= 0)
+	  {
+	    dw = 0;
+	  }
+	*rx = x + dw;
+      }
+
+    if (ry != NULL)
+      {
+	dh = mPagey * mZoomrate - mScrollwin->allocation.height;
+	dh = dh >> 1;
+	if (dh >= 0)
+	  {
+	    dh = 0;
+	  }
+	*ry = y + dh;
+      }
   }
 
   ApvlvDocCache::ApvlvDocCache (ApvlvFile * file)
   {
     mFile = file;
     mPagenum = -1;
+    mLines = NULL;
     mData = NULL;
     mSize = 0;
     mBuf = NULL;
@@ -1493,6 +1634,12 @@ namespace apvlv
     mZoom = zm;
     mRotate = rot;
 
+    if (mLines != NULL)
+      {
+	delete mLines;
+	mLines = NULL;
+      }
+
     if (mData != NULL)
       {
 	delete[]mData;
@@ -1539,6 +1686,7 @@ namespace apvlv
 					      ac->mWidth, ac->mHeight,
 					      3 * ac->mWidth,
 					      NULL, NULL);
+    debug ("ac->mFile: %p", ac->mFile);
     ac->mFile->render (ac->mPagenum, ac->mWidth, ac->mHeight, ac->mZoom,
 		       ac->mRotate, bu, (char *) dat);
     // backup the pixbuf data
@@ -1553,6 +1701,8 @@ namespace apvlv
 
     ac->mData = dat;
     ac->mBuf = bu;
+
+    ac->preparelines (0, 0, tpagex, tpagey);
   }
 
   ApvlvDocCache::~ApvlvDocCache ()
@@ -1562,10 +1712,20 @@ namespace apvlv
 	delete mLinks;
       }
 
+    if (mLines != NULL)
+      {
+	delete mLines;
+      }
+
     if (mData != NULL)
-      delete[]mData;
+      {
+	delete[]mData;
+      }
+
     if (mBuf != NULL)
-      g_object_unref (mBuf);
+      {
+	g_object_unref (mBuf);
+      }
   }
 
   guint ApvlvDocCache::getpagenum ()
@@ -1609,6 +1769,194 @@ namespace apvlv
     return mLinks;
   }
 
+  bool ApvlvDocCache::canselect ()
+  {
+    return mLines != NULL ? true : false;
+  }
+
+  ApvlvWord *ApvlvDocCache::getword (int x, int y)
+  {
+    ApvlvLine *line;
+
+    line = getline (x, y);
+    if (line != NULL)
+      {
+	vector < ApvlvWord >::iterator itr;
+	for (itr = line->mWords.begin (); itr != line->mWords.end (); itr++)
+	  {
+	    debug ("itr: %f,%f", itr->pos.y1, itr->pos.y2);
+	    if (x >= itr->pos.x1 && x <= itr->pos.x2)
+	      {
+		return &(*itr);
+	      }
+	  }
+      }
+    return NULL;
+  }
+
+  ApvlvLine *ApvlvDocCache::getline (double x, double y)
+  {
+    debug ("getline: %f", y);
+
+    if (mLines == NULL)
+      {
+	return NULL;
+      }
+
+    vector < ApvlvLine >::iterator itr;
+    for (itr = mLines->begin (); itr != mLines->end (); itr++)
+      {
+	debug ("itr: %f,%f", itr->pos.y1, itr->pos.y2);
+	if (y >= itr->pos.y2 && y <= itr->pos.y1)
+	  {
+	    return &(*itr);
+	  }
+      }
+    return NULL;
+  }
+
+  void ApvlvDocCache::preparelines (double x1, double y1, double x2,
+				    double y2)
+  {
+    if (strcmp (gParams->values ("doubleclick"), "page") == 0
+	|| strcmp (gParams->values ("doubleclick"), "none") == 0)
+      {
+	return;
+      }
+
+    gchar *content = NULL;
+    mFile->pagetext (mPagenum, x1, y1, x2, y2, &content);
+    if (content != NULL)
+      {
+	ApvlvPoses *results;
+	string word;
+
+	mLines = new vector < ApvlvLine >;
+
+	ApvlvPos lastpos = { 0, 0, 0, 0 };
+
+	if (strcmp (gParams->values ("doubleclick"), "word") == 0)
+	  {
+	    stringstream ss (content);
+	    while (!ss.eof ())
+	      {
+		ss >> word;
+		results = mFile->pagesearch (mPagenum, word.c_str ());
+		if (results != NULL)
+		  {
+		    lastpos = prepare_add (lastpos, results, word.c_str ());
+		    delete results;
+		  }
+	      }
+	  }
+	else
+	  {
+	    gchar **v, *p;
+	    int i;
+
+	    v = g_strsplit (content, "\n", -1);
+	    if (v != NULL)
+	      {
+		for (i = 0; v[i] != NULL; ++i)
+		  {
+		    p = v[i];
+		    while (*p != '\0' && isspace (*p))
+		      {
+			p++;
+		      }
+		    if (*p == '\0')
+		      {
+			continue;
+		      }
+		    debug ("search [%s]", p);
+		    results = mFile->pagesearch (mPagenum, p);
+		    if (results != NULL)
+		      {
+			lastpos = prepare_add (lastpos, results, p);
+			delete results;
+		      }
+		  }
+		g_strfreev (v);
+	      }
+	  }
+
+	vector < ApvlvLine >::iterator it;
+	for (it = mLines->begin (); it != mLines->end (); it++)
+	  {
+//              debug ("line: %f, %f, %f, %f", it->pos.x1, it->pos.y1,
+//                     it->pos.x2, it->pos.y2);
+	    vector < ApvlvWord >::iterator wit;
+	    for (wit = it->mWords.begin (); wit != it->mWords.end (); wit++)
+	      {
+//                 debug ("word: %f, %f, %f, %f: %s", wit->pos.x1, wit->pos.y1,
+//                         wit->pos.x2, wit->pos.y2, wit->word.c_str ());
+	      }
+	  }
+	g_free (content);
+      }
+  }
+
+  ApvlvPos ApvlvDocCache::prepare_add (ApvlvPos & lastpos,
+				       ApvlvPoses * results, const char *word)
+  {
+    ApvlvPoses::iterator itr;
+    for (itr = results->begin (); itr != results->end (); itr++)
+      {
+	itr->x1 *= mZoom;
+	itr->x2 *= mZoom;
+	itr->y1 = mHeight - itr->y1 * mZoom;
+	itr->y2 = mHeight - itr->y2 * mZoom;
+
+	if ((lastpos.y2 < itr->y2)
+	    || (lastpos.y2 - itr->y2 < 0.0001 && lastpos.x2 < itr->x2))
+	  {
+	    debug ("[%s] x1:%f, x2:%f, y1:%f, y2:%f", word, itr->x1,
+		   itr->x2, itr->y1, itr->y2);
+	    break;
+	  }
+      }
+
+    if (itr == results->end ())
+      {
+	itr = results->begin ();
+      }
+
+//      debug ("itr: %f, %f, %f, %f", itr->x1, itr->y1, itr->x2, itr->y2);
+    vector < ApvlvLine >::iterator litr;
+    for (litr = mLines->begin (); litr != mLines->end (); litr++)
+      {
+//          debug ("litr: %f, %f", litr->pos.y1, litr->pos.y2);
+	if (fabs (itr->y1 - litr->pos.y1) < 0.0001
+	    && fabs (itr->y2 - litr->pos.y2) < 0.0001)
+	  {
+	    break;
+	  }
+      }
+
+    ApvlvWord aword = { *itr, word };
+    if (litr != mLines->end ())
+      {
+	litr->mWords.push_back (aword);
+	if (itr->x1 < litr->pos.x1)
+	  {
+	    litr->pos.x1 = itr->x1;
+	  }
+	if (itr->x2 > litr->pos.x2)
+	  {
+	    litr->pos.x2 = itr->x2;
+	  }
+      }
+    else
+      {
+	ApvlvLine line;
+	line.pos = *itr;
+	line.mWords.push_back (aword);
+	mLines->push_back (line);
+      }
+
+    return *itr;
+  }
+
   ApvlvDocStatus::ApvlvDocStatus (ApvlvDoc * doc)
   {
     mDoc = doc;
@@ -1625,24 +1973,11 @@ namespace apvlv
 
   void ApvlvDocStatus::active (bool act)
   {
-    GdkColor c;
-
-    if (act)
-      {
-	c.red = 300;
-	c.green = 300;
-	c.blue = 300;
-      }
-    else
-      {
-	c.red = 30000;
-	c.green = 30000;
-	c.blue = 30000;
-      }
-
     for (unsigned int i = 0; i < AD_STATUS_SIZE; ++i)
       {
-	gtk_widget_modify_fg (mStlab[i], GTK_STATE_NORMAL, &c);
+	gtk_widget_modify_fg (mStlab[i],
+			      (act) ? GTK_STATE_ACTIVE :
+			      GTK_STATE_INSENSITIVE, NULL);
       }
   }
 
@@ -1665,11 +2000,6 @@ namespace apvlv
       {
 	gint pn = mDoc->pagenumber ();
 
-	if (mDoc->continuous () && mDoc->scrollrate () > 0.5)
-	  {
-	    pn++;
-	  }
-
 	char temp[AD_STATUS_SIZE][256];
 	gchar *bn;
 	bn = g_path_get_basename (mDoc->filename ());
