@@ -27,6 +27,7 @@
 /* @date Created: 2008/09/30 00:00:00 Alf */
 
 #include "ApvlvParams.hpp"
+#include "ApvlvInfo.hpp"
 #include "ApvlvDir.hpp"
 #include "ApvlvCmds.hpp"
 #include "ApvlvView.hpp"
@@ -49,8 +50,9 @@ namespace apvlv
 {
   ApvlvView *gView = NULL;
 
+  const int ApvlvView::APVLV_MENU_HEIGHT = 20;
   const int ApvlvView::APVLV_CMD_BAR_HEIGHT = 20;
-  const int ApvlvView::APVLV_TABS_HEIGHT = 30;
+  const int ApvlvView::APVLV_TABS_HEIGHT = 36;
 
     ApvlvView::ApvlvView (const char *filename):mCurrTabPos (-1)
   {
@@ -83,6 +85,15 @@ namespace apvlv
     mViewBox = gtk_vbox_new (FALSE, 0);
     gtk_container_add (GTK_CONTAINER (mMainWindow), mViewBox);
 
+    mMenu = new ApvlvMenu ();
+
+    if (strchr (gParams->values ("guioptions"), 'm') != NULL)
+      {
+	mMenu->setsize (mWidth, APVLV_MENU_HEIGHT - 1);
+	gtk_box_pack_start (GTK_BOX (mViewBox), mMenu->widget (), TRUE, TRUE,
+			    0);
+      }
+
     mTabContainer = gtk_notebook_new ();
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (mTabContainer), FALSE);
     gtk_notebook_set_scrollable (GTK_NOTEBOOK (mTabContainer), TRUE);
@@ -133,6 +144,7 @@ namespace apvlv
   {
     size_t i;
 
+    delete mMenu;
 
     for (int i = 0; i < (int) mTabList.size (); i++)
       {
@@ -173,32 +185,9 @@ namespace apvlv
 
   void ApvlvView::open ()
   {
-    const char *lastfile = NULL;
     gchar *dirname;
 
-    char *path = absolutepath (sessionfile.c_str ());
-    ifstream os (path, ios::in);
-    g_free (path);
-
-    string line, files;
-    if (os.is_open ())
-      {
-
-	while ((getline (os, line)) != NULL)
-	  {
-	    const char *p = line.c_str ();
-
-	    if (*p == '>')
-	      {
-		stringstream ss (++p);
-		ss >> files;
-		lastfile = files.c_str ();
-	      }
-	  }
-	os.close ();
-      }
-
-    GtkWidget *dia = gtk_file_chooser_dialog_new ("",
+    GtkWidget *dia = gtk_file_chooser_dialog_new ("Open ...",
 						  GTK_WINDOW (mMainWindow),
 						  GTK_FILE_CHOOSER_ACTION_SAVE,
 						  GTK_STOCK_CANCEL,
@@ -206,14 +195,14 @@ namespace apvlv
 						  GTK_STOCK_OK,
 						  GTK_RESPONSE_ACCEPT,
 						  NULL);
+    infofile *fp = gInfo->file (0);
     dirname =
-      lastfile ? g_dirname (lastfile) :
+      fp ? g_dirname (fp->file.c_str ()) :
       g_strdup (gParams->values ("defaultdir"));
-    gchar *realdir = absolutepath (dirname);
+    debug ("lastfile: [%s], dirname: [%s]", fp ? fp->file.c_str () : "",
+	   dirname);
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dia), dirname);
     g_free (dirname);
-    debug ("lastfile: [%s], dirname: [%s]", lastfile, realdir);
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dia), realdir);
-    g_free (realdir);
 
     GtkFileFilter *filter = gtk_file_filter_new ();
     gtk_file_filter_add_mime_type (filter, "PDF File");
@@ -237,31 +226,8 @@ namespace apvlv
 
   void ApvlvView::opendir ()
   {
-    const char *lastfile = NULL;
     gchar *dirname;
 
-    char *path = absolutepath (sessionfile.c_str ());
-    ifstream os (path, ios::in);
-    g_free (path);
-
-    string line, files;
-    if (os.is_open ())
-      {
-
-	while ((getline (os, line)) != NULL)
-	  {
-	    const char *p = line.c_str ();
-
-	    if (*p == '>')
-	      {
-		stringstream ss (++p);
-		ss >> files;
-		lastfile = files.c_str ();
-	      }
-	  }
-	os.close ();
-      }
-
     GtkWidget *dia = gtk_file_chooser_dialog_new ("",
 						  GTK_WINDOW (mMainWindow),
 						  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
@@ -270,8 +236,8 @@ namespace apvlv
 						  GTK_STOCK_OK,
 						  GTK_RESPONSE_ACCEPT,
 						  NULL);
-    dirname =
-      lastfile ? g_dirname (lastfile) :
+    infofile *fp = gInfo->file (0);
+    dirname = fp ? g_dirname (fp->file.c_str ()) :
       g_strdup (gParams->values ("defaultdir"));
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dia), dirname);
     g_free (dirname);
@@ -450,7 +416,7 @@ namespace apvlv
     mTabList.erase (remPos);
     if (c == (int) mTabList.size ())
       {
-	warnp ("erase failed to remove context");
+	gView->errormessage ("erase failed to remove context");
       }
 
     if ((int) mTabList.size () <= 1)
@@ -635,21 +601,45 @@ namespace apvlv
     char s[2] = { 0 };
     s[0] = ch;
     gtk_entry_set_text (GTK_ENTRY (mCommandBar), s);
-    cmd_show ();
+    cmd_show (CMD_CMD);
   }
 
   void ApvlvView::promptcommand (const char *s)
   {
     gtk_entry_set_text (GTK_ENTRY (mCommandBar), s);
-    cmd_show ();
+    cmd_show (CMD_CMD);
   }
 
-  void ApvlvView::cmd_show ()
+  void ApvlvView::errormessage (const char *str, ...)
+  {
+    gchar estr[512];
+    va_list vap;
+    va_start (vap, str);
+    vsnprintf (estr, sizeof estr, str, vap);
+    va_end (vap);
+    gtk_entry_set_text (GTK_ENTRY (mCommandBar), "ERROR: ");
+    gtk_entry_append_text (GTK_ENTRY (mCommandBar), estr);
+    cmd_show (CMD_MESSAGE);
+  }
+
+  void ApvlvView::infomessage (const char *str, ...)
+  {
+    gchar estr[512];
+    va_list vap;
+    va_start (vap, str);
+    vsnprintf (estr, sizeof estr, str, vap);
+    va_end (vap);
+    gtk_entry_set_text (GTK_ENTRY (mCommandBar), "INFO: ");
+    gtk_entry_append_text (GTK_ENTRY (mCommandBar), estr);
+    cmd_show (CMD_MESSAGE);
+  }
+
+  void ApvlvView::cmd_show (int cmdtype)
   {
     if (mMainWindow == NULL)
       return;
 
-    mHasCmd = TRUE;
+    mCmdType = cmdtype;
 
     mRootWindow->setsize (mWidth, adjheight ());
 
@@ -662,7 +652,7 @@ namespace apvlv
   {
     if (mMainWindow == NULL)
       return;
-    mHasCmd = FALSE;
+    mCmdType = CMD_NONE;
 
     gtk_widget_hide (mCommandBar);
     mRootWindow->setsize (mWidth, adjheight ());
@@ -737,9 +727,9 @@ namespace apvlv
       }
   }
 
-  ApvlvDoc *ApvlvView::crtadoc ()
+  ApvlvCore *ApvlvView::crtadoc ()
   {
-    return (ApvlvDoc *) currentWindow ()->getCore ();
+    return currentWindow ()->getCore ();
   }
 
   returnType ApvlvView::subprocess (int ct, guint key)
@@ -818,27 +808,32 @@ namespace apvlv
     return MATCH;
   }
 
-  void ApvlvView::run (const char *str)
+  bool ApvlvView::run (const char *str)
   {
+    bool ret;
+
     switch (*str)
       {
       case SEARCH:
 	crtadoc ()->markposition ('\'');
-	crtadoc ()->search (str + 1);
+	ret = crtadoc ()->search (str + 1);
 	break;
 
       case BACKSEARCH:
 	crtadoc ()->markposition ('\'');
-	crtadoc ()->search (str + 1, true);
+	ret = crtadoc ()->search (str + 1, true);
 	break;
 
       case COMMANDMODE:
-	runcmd (str + 1);
+	ret = runcmd (str + 1);
 	break;
 
       default:
+	ret = false;
 	break;
       }
+
+    return ret;
   }
 
   void ApvlvView::settitle (const char *title)
@@ -846,8 +841,10 @@ namespace apvlv
     gtk_window_set_title (GTK_WINDOW (mMainWindow), title);
   }
 
-  void ApvlvView::runcmd (const char *str)
+  bool ApvlvView::runcmd (const char *str)
   {
+    bool ret = true;
+
     if (*str == '!')
       {
 	apvlv_system (str + 1);
@@ -882,7 +879,19 @@ namespace apvlv
 	else if ((cmd == "o"
 		  || cmd == "open" || cmd == "doc") && subcmd != "")
 	  {
-	    loadfile (subcmd.c_str ());
+	    if (g_file_test (subcmd.c_str (), G_FILE_TEST_IS_DIR))
+	      {
+		ret = loaddir (subcmd.c_str ());
+	      }
+	    else if (g_file_test (subcmd.c_str (), G_FILE_TEST_EXISTS))
+	      {
+		ret = loadfile (subcmd.c_str ());
+	      }
+	    else
+	      {
+		errormessage ("no file '%s'", subcmd.c_str ());
+		ret = false;
+	      }
 	  }
 	else if (cmd == "TOtext" && subcmd != "")
 	  {
@@ -937,12 +946,12 @@ namespace apvlv
 	  }
 	else if ((cmd == "help" || cmd == "h") && subcmd == "setting")
 	  {
-	    crtadoc ()->loadfile (helppdf);
+	    crtadoc ()->loadfile (helppdf.c_str ());
 	    crtadoc ()->showpage (8);
 	  }
 	else if ((cmd == "help" || cmd == "h") && subcmd == "prompt")
 	  {
-	    crtadoc ()->loadfile (helppdf);
+	    crtadoc ()->loadfile (helppdf.c_str ());
 	    crtadoc ()->showpage (8);
 	  }
 	else if (cmd == "help" || cmd == "h")
@@ -1007,8 +1016,15 @@ namespace apvlv
 	      {
 		crtadoc ()->showpage (atoi (cmd.c_str ()) - 1);
 	      }
+	    else
+	      {
+		errormessage ("no command: '%s'", cmd.c_str ());
+		ret = false;
+	      }
 	  }
       }
+
+    return ret;
   }
 
   void
@@ -1031,7 +1047,7 @@ namespace apvlv
     ApvlvView::apvlv_view_keypress_cb (GtkWidget * wid, GdkEvent * ev,
 				       ApvlvView * view)
   {
-    if (view->mHasCmd == FALSE)
+    if (view->mCmdType == CMD_NONE)
       {
 	gCmds->append ((GdkEventKey *) ev);
 	return TRUE;
@@ -1044,7 +1060,7 @@ namespace apvlv
     ApvlvView::apvlv_view_commandbar_cb (GtkWidget * wid, GdkEvent * ev,
 					 ApvlvView * view)
   {
-    if (view->mHasCmd == true)
+    if (view->mCmdType == CMD_CMD)
       {
 	view->mInHistroy = false;
 
@@ -1055,12 +1071,24 @@ namespace apvlv
 	      (gchar *) gtk_entry_get_text (GTK_ENTRY (view->mCommandBar));
 	    if (str && strlen (str) > 0)
 	      {
-		view->run (str);
-		view->mCmdHistroy.push_back (str);
-		view->mCurrHistroy = view->mCmdHistroy.size () - 1;
+		if (view->run (str) == true)
+		  {
+		    view->mCmdHistroy.push_back (str);
+		    view->mCurrHistroy = view->mCmdHistroy.size () - 1;
+		    view->cmd_hide ();
+		    return TRUE;
+		  }
+		else
+		  {
+		    debug ("");
+		    return TRUE;
+		  }
+	      }
+	    else
+	      {
+		view->cmd_hide ();
+		return TRUE;
 	      }
-	    view->cmd_hide ();
-	    return TRUE;
 	  }
 	else if (gek->keyval == GDK_Tab)
 	  {
@@ -1099,9 +1127,8 @@ namespace apvlv
 	    view->mInHistroy = true;
 	    gtk_entry_set_text (GTK_ENTRY (view->mCommandBar),
 				view->mCurrHistroy > 0 ?
-				view->mCmdHistroy[view->
-						  mCurrHistroy--].c_str () :
-				view->mCmdHistroy[0].c_str ());
+				view->mCmdHistroy[view->mCurrHistroy--].
+				c_str () : view->mCmdHistroy[0].c_str ());
 	    return TRUE;
 	  }
 	else if (gek->keyval == GDK_Down)
@@ -1115,16 +1142,23 @@ namespace apvlv
 	    gtk_entry_set_text (GTK_ENTRY (view->mCommandBar),
 				(size_t) view->mCurrHistroy <
 				view->mCmdHistroy.size () -
-				1 ? view->mCmdHistroy[++view->
-						      mCurrHistroy].c_str () :
-				view->mCmdHistroy[view->mCmdHistroy.size () -
-						  1].c_str ());
+				1 ? view->mCmdHistroy[++view->mCurrHistroy].
+				c_str () : view->mCmdHistroy[view->
+							     mCmdHistroy.
+							     size () -
+							     1].c_str ());
 	    return TRUE;
 	  }
 
 
 	return FALSE;
       }
+    else if (view->mCmdType == CMD_MESSAGE)
+      {
+	debug ("");
+	view->cmd_hide ();
+	return TRUE;
+      }
 
     return FALSE;
   }
@@ -1152,10 +1186,17 @@ namespace apvlv
     int adj = 0;
     if (gtk_notebook_get_show_tabs (GTK_NOTEBOOK (mTabContainer)))
       adj += APVLV_TABS_HEIGHT;
-    if (mHasCmd)
+    if (mCmdType != CMD_NONE)
       adj += APVLV_CMD_BAR_HEIGHT;
 
-    return mHeight - adj;
+    if (strchr (gParams->values ("guioptions"), 'm') != NULL)
+      {
+	return mHeight - APVLV_MENU_HEIGHT - adj;
+      }
+    else
+      {
+	return mHeight - adj;
+      }
   }
 
   void ApvlvView::switchtab (int tabPos)
@@ -1188,6 +1229,8 @@ namespace apvlv
     else
       gfilename = g_path_get_basename (filename);
 
+    settitle (gfilename);
+
     if (mTabList[mCurrTabPos].numwindows > 1)
       g_snprintf (tagname, sizeof tagname, "[%d] %s",
 		  mTabList[mCurrTabPos].numwindows, gfilename);
